//battle_status.js
let battleStatus = null;

async function getBattleStatus() {
    const currentUrl = window.location.href;
    const now = new Date().getTime(); // 現時刻

    // urlが決戦画面の場合は最新値取得⇒ストレージ更新
    //console.log("getBattleStatus:", document.readyState);
    if (document.readyState != 'loading' &&
        (currentUrl.includes("battle_top_sp.aspx") ||
            currentUrl.includes("192.168.2.150")))  // debug
    {
        //console.log("決戦画面ルート");
        const { kz1, kz2 } = getKz();
        battleStatus = {
            fb: getFB(),
            score1: getScore(1),
            score2: getScore(2),
            pscore: getPersonalScore(),
            tp: getTP(),
            ogi1: getOgi("own"),
            ogi2: getOgi("enemy"),
            ogiTime1: getOgiTime1(),
            ogiRemain1: Math.max(0, Math.floor(getOgiTime1() - now / 1000)),
            ogiTime2: getOgiTime2(),
            ogiRemain2: Math.max(0, Math.floor(getOgiTime2() - now / 1000)),
            kz1: kz1,
            kz2: kz2,
            lastupdate: Math.floor(now / 1000)
        };
        saveBattleStatus(battleStatus);
        return battleStatus;
    } else {
        // urlが決戦画面以外の場合は、ストレージから読み出す
        let battleStatus = await loadBattleStatus();
        if (battleStatus && battleStatus.ogiTime1 < now / 1000) {
            clearBattleStatus();
            return null;
        }

        // デバッグ用
        if (0) {
            console.log("debug getBS");
            battleStatus = {
                fb: "後衛",
                score1: "★123 456",
                score2: "★456 456",
                pscore: "★2 234",
                tp: 25
            }
            return battleStatus;
        }

        return battleStatus;
    }
}


async function loadBattleStatus() {
    return new Promise((resolve) => {
        chrome.storage.local.get(['battleStatus'], function (result) {
            const loadedData = result['battleStatus'];
            //console.log("battleStatus loaded from local storage:", loadedData);
            resolve(loadedData);
        });
    });
}


function saveBattleStatus(battleStatus) {
    new Promise((resolve) => {
        chrome.storage.local.set({ 'battleStatus': battleStatus }, function () {
            //console.log(battleStatus);
            resolve();
        });
    });
}


// 初期化関数
async function clearBattleStatus() {
    console.log("決戦状態初期化");
    battleStatus = null;
    // ストレージのデータも初期化
    await chrome.storage.local.set({ 'battleStatus': null }, function () {
        console.log("battleStatus cleared in local storage");
    });
}


function getScore(index) {
    const vsBox = document.querySelector('.vs-box');
    if (vsBox) {
        const liElement = vsBox.querySelector(`li:nth-child(${index})`);

        if (liElement) {
            let scoreText = 0;
            const scoreElement = liElement.querySelector('.score');
            if (scoreElement) {
                scoreText = scoreElement ? scoreElement.textContent.trim().replace(/,/g, '') : '';
            } else {
                return;
            }
            const starText = liElement.querySelector('.star') ? liElement.querySelector('.star').textContent.trim().replace(/,/g, '').replace("x ", "★") : '';
            if (starText) {
                return `${starText} ${scoreText}`;
            }
            return scoreText;
        }
        return;
    }
}


function getPersonalScore() {
    const powerBox = document.querySelector('.power-box');
    //console.log("powerBox:", powerBox);
    if (powerBox) {
        let numberText = powerBox.lastElementChild.childNodes[0].data;
        numberText = numberText.replace(/×/, "").replace(/,/g, '');
        //console.log("numberText:", numberText)
        return numberText;
    }
    return null;
}


// 前後衛判定
function getFB() {
    const divElement = document.querySelectorAll('div > p:first-child');

    // divElementが存在し、少なくとも2つの要素を持っている場合に処理を進める
    if (divElement && divElement.length >= 2) {
        //console.log("divElement[1].textContent.trim():", divElement[1].textContent.trim());
        if (divElement[1].textContent.trim().includes("計略")) {
            return "前衛";
        } else {
            return "後衛";
        }
    }
    // 上記条件に合致しない場合
    return '';
}


function getTP() {
    const rtbValue = document.querySelectorAll('.rtb-value');
    if (!rtbValue[1]) {
        return 0; // 対象の要素が存在しない場合の処理
    }
    const tpText = rtbValue[1].textContent.trim().match(/TP (\d+)\/\d+/);
    const tpValue = tpText ? tpText[1] : '';
    return tpValue;
}


function getOgi(side) {
    const ownOugiComment = document.querySelector(`#${side}-ougi-comment1`);
    let iconElement = document.querySelector(`.${side}-ougi-comment-bar .icon-mving`) || document.querySelector(`.${side}-ougi-comment-bar .icon-stnby`);

    if (iconElement && ownOugiComment) {
        let iconText = iconElement.textContent.trim();
        let commentText = ownOugiComment.querySelector('div').textContent.trim().replace(/(\r\n|\n|\r)/gm, "");
        commentText = commentText.replace(/.+「/, "");
        commentText = commentText.replace(/」.+/, "");
        return `${iconText}:${commentText}`;
    }
    return "";
}


function getOgiTime1() {
    if (document.body.innerHTML) {
        // 日時が記述されている部分を正規表現で検索
        const regexResult = document.body.innerHTML.match(/own-ougi-countdown1.+new Date\('([^']+)'\)/);
        if (regexResult && regexResult[1]) {
            const dateTimeString = regexResult[1];
            // 日時文字列を Date オブジェクトに変換
            const dateTimeObject = new Date(dateTimeString);
            // Date オブジェクトを秒数に変換して返す
            const seconds = Math.floor(dateTimeObject.getTime() / 1000);
            return seconds;
        }
    }
    return null; // 適切な日時が見つからない場合
}


function getOgiTime2() {
    const regexResult = document.body.innerHTML.match(/enemy-ougi-countdown1.+new Date\('([^']+)'\)/);
    if (regexResult && regexResult[1]) {
        const dateTimeString = regexResult[1];
        const dateTimeObject = new Date(dateTimeString);
        const seconds = Math.floor(dateTimeObject.getTime() / 1000);
        return seconds;
    }
    return null;
}


function getOgiTimeDifference(targetElementId) {
    const regexResult = document.body.innerHTML.match(new RegExp(`${targetElementId}.+new Date\\('([^']+)'\)`));
    if (regexResult && regexResult[1]) {
        const dateTimeString = regexResult[1];
        const targetDateTime = new Date(dateTimeString).getTime();
        const now = new Date().getTime();
        // 時刻の差を秒数で計算
        const differenceInSeconds = Math.floor((targetDateTime - now) / 1000);
        return differenceInSeconds;
    }
    return null; // 適切な日時が見つからない場合
}


function getKz() {
    let kz1, kz2;
    // スクリプト要素を取得
    const scriptElements = document.querySelectorAll('script[type="text/javascript"]');

    // スクリプト内のコードを確認
    scriptElements.forEach((scriptElement) => {
        const scriptContent = scriptElement.textContent;
        // window.pexOwnの情報を取得
        if (scriptContent.includes("window.pexOwn")) {
            const matchResult = scriptContent.match(/pexOwn.+%26uc1%3D\d(\d)%26uc2%3D\d(\d)%26uc3%3D\d(\d)%26uc4%3D\d(\d)%26uc5%3D\d(\d)/);
            if (matchResult) {
                kz1 = matchResult.slice(1).map(Number).reduce((sum, num) => sum + num, 0);
            }
        }
        // window.pexEnemyの情報を取得
        if (scriptContent.includes("window.pexEnemy")) {
            const matchResult = scriptContent.match(/pexEnemy.+%26uc1%3D\d(\d)%26uc2%3D\d(\d)%26uc3%3D\d(\d)%26uc4%3D\d(\d)%26uc5%3D\d(\d)/);
            if (matchResult) {
                kz2 = matchResult.slice(1).map(Number).reduce((sum, num) => sum + num, 0);
            }
        }
    });
    //console.log(`kz=${kz1},${kz2}`);
    return { kz1, kz2 };
}


function getKzStatus() {
    const statusElement = document.querySelector('.mystatus-box.rtb-status li span[style="color:#FF0000"]');
    console.log("getKz", statusElement);
    if (statusElement && statusElement.textContent.trim() === '気絶中') {
        return '気絶中';
    }
    return null;
}

function checkCombo() {

}

