//const BASE_URL = "http://rise.3594.pgw.jp";
const currentUrl = new URL(window.location.href);

// URLパラメータからsettingIdを取得
const settingId = currentUrl.searchParams.get("settingId");
console.log("Setting ID:", settingId);

const DEFAULT_SETTINGS = {
    settingName: "設定1",
    battleType:  "battle",
    buttonCountB: 5,
    buttonCountL: 10,
    buttonCountR: 10,
    buttonCountA: 50,
    buttonCountO: 3,
    executeInterval: 1000,
    waitTime: 300,
    tpThreshold: 30,
    recoveryOption: "on",
    recoveryUrl: ""
};


document.addEventListener("DOMContentLoaded", function () {
    const RADIO_OPTIONS = ["回復ON", "回復OFF"];

    // ラジオボタンの初期設定
    async function initRadioButtons() {
        const recoveryOptionRadioGroup = document.getElementById("recoveryOptionRadioGroup");

        // Storage から recoveryOption を取得
        const { recoveryOption } = await getStorage(settingId, { recoveryOption: DEFAULT_SETTINGS.recoveryOption });

        RADIO_OPTIONS.forEach((label, index) => {
            const value = index === 0 ? "on" : "off";
            const radio = createElement("input", {
                type: "radio",
                name: "recoveryOption",
                value: value,
            });
            if (recoveryOption === value) {
                radio.checked = true;
            }

            const labelElement = createElement("label", {}, label);

            const row = createElement("tr");
            const radioCell = createElement("td");
            const labelCell = createElement("td");
            radioCell.appendChild(radio);
            labelCell.appendChild(labelElement);
            row.appendChild(radioCell);
            row.appendChild(labelCell);
            recoveryOptionRadioGroup.appendChild(row);

            // ラジオボタンが変更されたときのイベントリスナーを追加
            radio.addEventListener("change", async function () {
                await setStorage({ recoveryOption: value });
            });
        });
    }


    // 設定保存ボタン
    document.getElementById("saveButton").addEventListener("click", async function () {
        // 設定値の取得
        const settingName = document.getElementById("settingName").value;
        const userName = document.getElementById("userName").value; // ユーザ名の追加
        const recoveryOption = document.querySelector(
            'input[name="recoveryOption"]:checked'
        ).value;
        const executeInterval = document.getElementById("executeInterval").value;
        const waitTime = document.getElementById("waitTime").value;
        const recoveryUrl = document.getElementById("recoveryUrl").value;
        const battleType = document.getElementById("battleType").value;
        const buttonSettingsB = [];
        const buttonSettingsL = [];
        const buttonSettingsR = [];
        const buttonSettingsA = [];
        const buttonSettingsO = [];
        const buttonSize = document.getElementById("buttonSizeSelect").value;

        // 入力されたボタン情報を設定オブジェクトに追加する関数
        const processButtonInputs = (buttonType, targetButtonSettings) => {
            const buttonCount = getButtonCount(buttonType);
            console.log(`buttonCount:${buttonCount}, type=${buttonType}`);
            for (let i = 1; i <= buttonCount; i++) {
                const textElement = document.getElementById(`button${buttonType}${i}Text`);
                const buttonText = textElement && textElement.value
                    ? textElement.value : `リンク${buttonType}${i}`;

                const propertyElement = document.getElementById(`button${buttonType}${i}Property`);
                const buttonProperty = propertyElement && propertyElement.value
                    ? propertyElement.value : 0;

                const urlElement = document.getElementById(`button${buttonType}${i}Url`);
                const buttonUrl = urlElement && urlElement.value
                    ? urlElement.value : "";

                const ggsgElement = document.getElementById(`button${buttonType}${i}GGSG`);
                const buttonGGSG = ggsgElement && ggsgElement.value
                    ? ggsgElement.value : "";

                // 種別により文字色・背景色・枠線を変更
                // [枠:前衛] 攻撃:青、計略:緑、回復:紫
                // [枠:後衛] 応援:青、弓:緑、攻撃:紫
                // [背景] 通常: 青、滅: 紫
                // 滅
                // 応援/弓・攻撃

                // targetButtonSettings に保存
                targetButtonSettings.push({ type: buttonType, text: buttonText, property: buttonProperty, url: buttonUrl, ggsg: buttonGGSG });
            }
        };

        // 各ボタンタイプについて設定を保存
        const buttonCountB = getButtonCount("B");
        const buttonCountL = getButtonCount("L");
        const buttonCountR = getButtonCount("R");
        const buttonCountA = getButtonCount("A");
        const buttonCountO = getButtonCount("O");

        processButtonInputs("B", buttonSettingsB);
        processButtonInputs("L", buttonSettingsL);
        processButtonInputs("R", buttonSettingsR);
        processButtonInputs("A", buttonSettingsA);
        processButtonInputs("O", buttonSettingsO);
        //console.log(buttonSettingsA);

        // 設定をストレージに保存
        await setStorage(settingId, {
            settingName: settingName,
            userName: userName, // ユーザ名の追加
            battleType: battleType,
            buttonCountB: buttonCountB,
            buttonCountL: buttonCountL,
            buttonCountR: buttonCountR,
            buttonCountA: buttonCountA,
            buttonCountO: buttonCountO,
            executeInterval: executeInterval,
            waitTime: waitTime,
            buttonSettingsB: buttonSettingsB,
            buttonSettingsL: buttonSettingsL,
            buttonSettingsR: buttonSettingsR,
            buttonSettingsA: buttonSettingsA,
            buttonSettingsO: buttonSettingsO,
            buttonSize: buttonSize,
            recoveryOption: recoveryOption,
            recoveryUrl: recoveryUrl,
        });

        displayMessage("保存しました", true);
    });


    // ボタンサイズの設定の初期化
    async function initButtonSize() {
        const buttonSizeSelect = document.getElementById("buttonSizeSelect");

        // Storage から buttonSize を取得
        const { buttonSize } = await getStorage(settingId, { buttonSize: "medium" });

        // セレクトボックスの初期値設定
        if (buttonSize) {
            buttonSizeSelect.value = buttonSize;
        }
    }

    // ボタンサイズの初期化関数を呼び出し
    initButtonSize();


    // UI初期化関数
    async function initUI() {
        const data = await getStorage(settingId, {
            settingName: "設定1",  // 設定名の初期値を指定
            battleType: DEFAULT_SETTINGS.battleType,
            buttonCountB: DEFAULT_SETTINGS.buttonCountB,
            buttonCountL: DEFAULT_SETTINGS.buttonCountL,
            buttonCountR: DEFAULT_SETTINGS.buttonCountR,
            buttonCountA: DEFAULT_SETTINGS.buttonCountA,
            buttonCountO: DEFAULT_SETTINGS.buttonCountO,
            executeInterval: DEFAULT_SETTINGS.executeInterval,
            waitTime: DEFAULT_SETTINGS.waitTime,
            tpThreshold: DEFAULT_SETTINGS.tpThreshold,
            recoveryOption: DEFAULT_SETTINGS.recoveryOption,
            recoveryUrl: DEFAULT_SETTINGS.recoveryUrl,
            buttonSettingsB: getDefaultButtonSettings().filter(setting => setting.type === 'B'),
            buttonSettingsL: getDefaultButtonSettings().filter(setting => setting.type === 'L'),
            buttonSettingsR: getDefaultButtonSettings().filter(setting => setting.type === 'R'),
            buttonSettingsA: getDefaultButtonSettings().filter(setting => setting.type === 'A'),
            buttonSettingsO: getDefaultButtonSettings().filter(setting => setting.type === 'O')
        });
        document.getElementById("settingName").value = data.settingName;
        document.getElementById("battleType").value = data.battleType;
        document.getElementById("buttonCountB").value = data.buttonCountB;
        document.getElementById("buttonCountL").value = data.buttonCountL;
        document.getElementById("buttonCountR").value = data.buttonCountR;
        document.getElementById("buttonCountA").value = data.buttonCountA;
        document.getElementById("buttonCountO").value = data.buttonCountO;
        document.getElementById("executeInterval").value = data.executeInterval;
        document.getElementById("waitTime").value = data.waitTime;
        document.getElementById("tpThreshold").value = data.tpThreshold;
        

        //console.log(data.buttonSettingsA);

        ["B", "L", "R", "A", "O"].forEach((buttonType) => {
            generateButtonInputs(
                buttonType,
                data[`buttonCount${buttonType}`],
                data[`buttonSettings${buttonType}`]
            );
        });

        generateScenarioInputs(data[`buttonCountA`]);

        const { buttonSize } = await getStorage(settingId, { buttonSize: "medium" });
        document.getElementById("buttonSizeSelect").value = buttonSize;
    }

    // ラジオボタンの初期化
    initRadioButtons();

    // メインプログラムの開始点
    initUI();
});


function getDefaultButtonSettings() {
    return Array.from({ length: 10 }, (_, i) => ({
        type: "B",
        text: `B${i + 1}`,
        url: "",
    })).concat(
        Array.from({ length: 10 }, (_, i) => ({
            type: "L",
            text: `L${i + 1}`,
            url: "",
        }))
    ).concat(
        Array.from({ length: 10 }, (_, i) => ({
            type: "R",
            text: `R${i + 1}`,
            url: "",
        }))
    ).concat(
        Array.from({ length: 10 }, (_, i) => ({
            type: "A",
            text: `A${i + 1}`,
            url: "",
        }))

    ).concat(
        Array.from({ length: 10 }, (_, i) => ({
            type: "O",
            text: `O${i + 1}`,
            url: "",
        }))
    )

}


// 要素の作成
function createElement(tag, attributes, text) {
    const element = document.createElement(tag);
    for (const key in attributes) {
        element.setAttribute(key, attributes[key]);
    }
    if (text) {
        element.textContent = text;
    }
    return element;
}


// ボタン入力エリアの生成
function generateButtonInputs(buttonType, buttonCount, buttonSettings) {
    console.log(`buttonType: ${buttonType}, buttonSettings:`, buttonSettings);

    const buttonTextInputs = document.getElementById(`buttonTextInputs${buttonType}`);
    if (!buttonTextInputs) {
        console.error(`buttonTextInputs${buttonType} not found.`);
        return;
    }

    const filteredSettings = buttonSettings.filter((setting) => setting.type === buttonType);
    buttonTextInputs.innerHTML = "";

    for (let i = 1; i <= buttonCount; i++) {
        const setting = filteredSettings[i - 1];

        const initialText = setting ? setting.text : `${buttonType}${i}`;
        const initialUrl = setting ? setting.url : "";
        const initialGGSG = setting ? setting.ggsg : "";

        const labelCell = createElement("td", { style: "font-weight: bold;" }, `ボタン${buttonType}${i}`);

        const inputText = createElement("input", {
            id: `button${buttonType}${i}Text`,
            type: "text",
        });
        inputText.value = initialText;
        inputText.style.width = "8em"; // テキストボックス

        // ボタンプロパティ
        const property = createElement("input", {
            id: `button${buttonType}${i}Property`,
            type: "text",
        });
        if (setting && setting.property) {
            //console.log(`${buttonType}${i}: setting.property:`, setting.property)
            property.value = setting.property ? setting.property : 0;
        }
        property.style.width = "5em"; // テキストボックス

        const upButton = createElement("button", { class: "up" }, "▲");
        const downButton = createElement("button", { class: "down" }, "▼");

        upButton.addEventListener("click", function () {
            moveButtonSet(buttonType, "up", i, buttonCount);
        });

        downButton.addEventListener("click", function () {
            moveButtonSet(buttonType, "down", i, buttonCount);
        });

        const row = createElement("tr");
        row.appendChild(labelCell);
        labelCell.appendChild(inputText); // テキストボックスをラベルセルに追加
        labelCell.appendChild(property);
        labelCell.appendChild(upButton);
        labelCell.appendChild(downButton);

        // 奥義の場合はGGSG
        if (buttonType === "O") {
            const ggsg = createElement("hidden", {
                id: `button${buttonType}${i}GGSG`,
                type: "text",
            });
            ggsg.value = initialGGSG;
            labelCell.appendChild(ggsg);
            labelCell.classList.add(getGGSG(ggsg));
        }
        buttonTextInputs.appendChild(row);

        // URLの行
        const inputUrl = createElement("textarea", {
            id: `button${buttonType}${i}Url`,
            //rows: "3",
            cols: "20",
        });
        inputUrl.textContent = initialUrl;
        const urlCell = createElement("td");
        urlCell.appendChild(inputUrl);
        const urlRow = createElement("tr");
        urlRow.appendChild(urlCell);
        buttonTextInputs.appendChild(urlRow);
    }
}


// ボタン入力エリアの生成(シナリオ)
function generateScenarioInputs(buttonCount) {
    console.log("generateScenarioInputs:", buttonCount);
    const buttonType = "S";

    const buttonTextInputs = document.getElementById(`buttonTextInputs${buttonType}`);
    if (!buttonTextInputs) {
        console.error(`buttonTextInputs${buttonType} not found.`);
        return;
    }

    buttonTextInputs.innerHTML = "";
    for (let i = 1; i <= buttonCount; i++) {
        for (let j = 1; j <= 5; j++) {

            const labelCell = createElement("td", { style: "font-weight: bold;" }, `ボタン${buttonType}${i}`);


            const inputText = createElement("input", {
                id: `button${buttonType}${i}Text${j}`,
                type: "text",
            });
            inputText.value = `${buttonType}${i}`;
            inputText.style.width = "8em"; // テキストボックス
       
            const row = createElement("tr");
            row.appendChild(labelCell);
            labelCell.appendChild(inputText); // テキストボックスをラベルセルに追加
        }
    }
}


// 設定アップロード、ダウンロード
document.getElementById("loadSettingsButton").addEventListener("click", function () {
    const configId = document.getElementById("configIdInput").value;
    const url = `${BASE_URL}/rise/download/${configId}`;

    // 設定の読み込み
    loadSettingsFromCsvUrl(url);
    displayMessage(`設定ID:${configId}をダウンロードしました`, true);
});


document.getElementById("uploadSettingsButton").addEventListener("click", async function () {
    const url = `${BASE_URL}/rise/upload`;

    // 設定の取得
    const settings = await getStorage(settingId, {
        settingName: "設定1",
        userName: "",
        battleType: DEFAULT_SETTINGS.battleType,
        buttonCountB: DEFAULT_SETTINGS.buttonCountB,
        buttonCountL: DEFAULT_SETTINGS.buttonCountL,
        buttonCountR: DEFAULT_SETTINGS.buttonCountR,
        buttonCountA: DEFAULT_SETTINGS.buttonCountA,
        buttonCountO: DEFAULT_SETTINGS.buttonCountO,
        executeInterval: DEFAULT_SETTINGS.executeInterval,
        waitTime: DEFAULT_SETTINGS.waitTime,
        tpThreshold: DEFAULT_SETTINGS.tpThreshold,
        recoveryOption: DEFAULT_SETTINGS.recoveryOption,
        recoveryUrl: DEFAULT_SETTINGS.recoveryUrl,
        buttonSettingsB: getDefaultButtonSettings().filter(setting => setting.type === 'B'),
        buttonSettingsL: getDefaultButtonSettings().filter(setting => setting.type === 'L'),
        buttonSettingsR: getDefaultButtonSettings().filter(setting => setting.type === 'R'),
        buttonSettingsA: getDefaultButtonSettings().filter(setting => setting.type === 'A'),
        buttonSettingsO: getDefaultButtonSettings().filter(setting => setting.type === 'O'),
    });

    settings.userName = document.getElementById("userName").value;
    console.log("userName:", settings.userName);
    // ユーザ名が空欄の場合はエラーメッセージを表示
    if (!settings.userName.trim()) {
        displayMessage("アップロード時は、ユーザ名を入力してください", false);
        return;
    }

    // waitTime、buttonCountB、buttonCountL、buttonCountR を settings オブジェクトに含める
    settings.executeInterval = document.getElementById("executeInterval").value;
    settings.waitTime = document.getElementById("waitTime").value;
    settings.tpThreshold = document.getElementById("tpThreshold").value;
    settings.battleType = document.getElementById("battleType").value;
    settings.buttonCountB = getButtonCount("B");
    settings.buttonCountL = getButtonCount("L");
    settings.buttonCountR = getButtonCount("R");
    settings.buttonCountA = getButtonCount("A");
    settings.buttonCountO = getButtonCount("O");
    await setStorage(settingId, settings);

    //　アップロード時間
    settings.updateDate = new Date().toLocaleString();

    // 設定のアップロード
    saveButtonSettingsToCsv(url, settings);
});


// ボタンの数を取得する関数
const getButtonCount = (buttonType) => {
    const buttonCountElement = document.getElementById(`buttonCount${buttonType}`);
    return buttonCountElement ? parseInt(buttonCountElement.value, 10) : 0;
};


// ボタン設定データをURLにCSV形式で保存する関数
function saveButtonSettingsToCsv(url, settings) {
    const csvText = "config_id,setting_type,setting_key,setting_value\n" +
        Object.entries(settings)
            .map(([key, value]) => {
                if (key === 'settingName') {
                    return `1,property,${key},${value}`;
                }
                if (typeof value === 'object') {
                    return value.map((setting, index) => {
                        return `1,button,${key}${index + 1}Text,${setting.text}\n1,button,${key}${index + 1}Url,${setting.url}`;
                    }).join('\n');
                } else {
                    return `1,property,${key},${value}`;
                }
            })
            .join("\n");

    // BOM つきの UTF-8 ファイル生成
    const blob = createBOM_UTF8(csvText);

    const formData = new FormData();
    formData.append('file', blob, 'settings.csv');

    const xhr = new XMLHttpRequest();
    xhr.open('POST', url);
    xhr.send(formData);

    xhr.onload = function () {
        if (xhr.status === 200) {
            console.log('Settings saved in CSV format to server');
        } else {
            console.error('Failed to save settings:', xhr.statusText);
        }
    };

    xhr.onerror = function () {
        console.error('There was a problem with the XMLHttpRequest');
    };
}


// CSVをパースしてネストされたオブジェクトのリストに変換する関数
function parseCsv(text, delimiter = ",") {
    const rows = text.split("\n").filter((row) => row.trim().length > 0);
    const header = rows.shift().split(delimiter).map(s => s.trim());
    const data = rows.map((row) =>
        row.split(delimiter).reduce((acc, cell, i) => {
            acc[header[i]] = cell.trim().replace(/\r$/, "");
            return acc;
        }, {})
    );
    return data;
}


function loadSettingsFromCsvUrl(url) {
    return fetch(url)
        .then((response) => {
            if (!response.ok) {
                throw new Error("Network response was not ok");
            }
            return response.text();
        })
        .then((csvText) => {
            const settings = parseCsv(csvText);
            console.log("parseCSV", settings);

            // 新しい設定オブジェクトを構築します
            const properties = settings
                .filter((setting) => setting.setting_type === "property")
                .reduce((acc, property) => {
                    acc[property.setting_key] = property.setting_value;
                    return acc;
                }, {});

            // 設定名が存在するか確認
            const settingName = properties.settingName || '';

            // 設定名を表示
            console.log("Setting Name: ", settingName);

            const buttonSettings = settings.filter(
                (setting) => setting.setting_type === "button"
            );

            const buttonSettingsMap = buttonSettings.reduce((acc, setting) => {
                const type = setting.setting_key.charAt(6);
                const indexStr = setting.setting_key.match(/(\d+)/);
                const index = indexStr ? parseInt(indexStr[0], 10) - 1 : -1;
                const key = `${type}${index}`;

                if (index < 0) {
                    return acc;
                }

                if (!acc[type]) {
                    acc[type] = {};
                }

                if (!acc[type][index]) {
                    acc[type][index] = {
                        type: type,
                        text: "",
                        //property: 0,
                        url: "",
                    };
                }

                if (setting.setting_key.endsWith("Text")) {
                    acc[type][index].text = setting.setting_value;
                } else if (setting.setting_key.endsWith("Url")) {
                    acc[type][index].url = setting.setting_value;
                }

                return acc;
            }, {});

            const buttonSettingsList = [
                ...Object.values(buttonSettingsMap.B || {}),
                ...Object.values(buttonSettingsMap.L || {}),
                ...Object.values(buttonSettingsMap.R || {}),
                ...Object.values(buttonSettingsMap.A || {}),
            ];

            console.log("properties", properties);
            console.log("buttonSettingsList: ", buttonSettingsList);

            const newSettings = {
                settingName: settingName, // 設定名を追加
                battleType: properties.battleType,
                buttonCountB: parseInt(properties.buttonCountB, 10),
                buttonCountL: parseInt(properties.buttonCountL, 10),
                buttonCountR: parseInt(properties.buttonCountR, 10),
                buttonCountA: parseInt(properties.buttonCountA, 10),
                buttonCountO: parseInt(properties.buttonCountO, 10),
                executeInterval: parseInt(properties.executeInterval, 10),
                waitTime: parseInt(properties.waitTime, 10),
                tpThreshold: parseInt(properties.tpThreshold, 30),
                recoveryOption: properties.recoveryOption,
                recoveryUrl: properties.recoveryUrl,
                buttonSettings: buttonSettingsList,
                buttonSettingsB: buttonSettingsList.filter(setting => setting.type === "B"),
                buttonSettingsL: buttonSettingsList.filter(setting => setting.type === "L"),
                buttonSettingsR: buttonSettingsList.filter(setting => setting.type === "R"),
                buttonSettingsA: buttonSettingsList.filter(setting => setting.type === "A"),
                buttonSettingsO: buttonSettingsList.filter(setting => setting.type === "O"),
            };

            // ここで新しい設定オブジェクトをRestore関数に渡します
            restoreSettings(newSettings);

        })
        .catch((error) => {
            console.error("There was a problem with the fetch operation:", error);
        });
}


function restoreSettings(settings) {
    const { settingName, battleType,
        buttonCountB, buttonCountL, buttonCountR, buttonCountA, buttonCountO,
        executeInterval, waitTime, tpThreshold, recoveryOption, recoveryUrl } = settings;
    document.getElementById("settingName").value = settingName;
    document.getElementById("battleType").value = battleType;
    document.getElementById("buttonCountB").value = buttonCountB;
    document.getElementById("buttonCountL").value = buttonCountL;
    document.getElementById("buttonCountR").value = buttonCountR;
    document.getElementById("buttonCountA").value = buttonCountA;
    document.getElementById("buttonCountO").value = buttonCountO;
    document.getElementById("executeInterval").value = executeInterval;
    document.getElementById("waitTime").value = waitTime;
    document.getElementById("tpThreshold").value = tpThreshold;

    // ボタン設定を分割する
    const buttonSettingsB = settings.buttonSettings.filter(setting => setting.type === "B");
    const buttonSettingsL = settings.buttonSettings.filter(setting => setting.type === "L");
    const buttonSettingsR = settings.buttonSettings.filter(setting => setting.type === "R");
    const buttonSettingsA = settings.buttonSettings.filter(setting => setting.type === "A");
    const buttonSettingsO = settings.buttonSettings.filter(setting => setting.type === "O");
    //console.log("buttonSettingsO:", buttonSettingsO);

    ["B", "L", "R", "A", "O"].forEach((buttonType) => {
        generateButtonInputs(
            buttonType,
            settings[`buttonCount${buttonType}`],
            settings[`buttonSettings${buttonType}`]
        );
    });



    // ラジオボタンの初期化
    const recoveryOptionRadios = document.querySelectorAll('input[name="recoveryOption"]');
    recoveryOptionRadios.forEach((radio) => {
        if (radio.value === recoveryOption) {
            radio.checked = true;
        } else {
            radio.checked = false;
        }
    });

    document.getElementById("recoveryUrl").value = recoveryUrl;
}


// 入力された設定データをURLにCSV形式で保存する関数
function getAbilityInfoFromUrl(url, settings) {
    const csvText = "aid\n" +
        Object.entries(settings)
            .map(([key, value]) => {
                if (key === 'settingName') {
                    return `1,property,${key},${value}`;
                }
                if (typeof value === 'object') {
                    return value.map((setting, index) => {
                        return `1,button,${key}${index + 1}Text,${setting.text}\n1,button,${key}${index + 1}Url,${setting.url}`;
                    }).join('\n');
                } else {
                    return `1,property,${key},${value}`;
                }
            })
            .join("\n");

    // BOM つきの UTF-8 ファイル生成
    const blob = createBOM_UTF8(csvText);

    const formData = new FormData();
    formData.append('file', blob, 'settings.csv');

    const xhr = new XMLHttpRequest();
    xhr.open('POST', url);
    xhr.send(formData);

    xhr.onload = function () {
        if (xhr.status === 200) {
            console.log('Settings saved in CSV format to server');
        } else {
            console.error('Failed to save settings:', xhr.statusText);
        }
    };

    xhr.onerror = function () {
        console.error('There was a problem with the XMLHttpRequest');
    };
}


// ストレージから設定を取得する関数
async function getStorage(settingId, keysAndDefaults) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.get({ [storageKey]: keysAndDefaults }, (obj) => {
            resolve(obj[storageKey]);
        });
    });
}

// ストレージに設定を保存する関数
async function setStorage(settingId, data) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.set({ [storageKey]: data }, () => {
            console.log(`Settings saved for ID ${settingId}:`, data);
            resolve();
        });
    });
}

// ストレージから設定を取得する関数
async function getStorage2(settingId, keysAndDefaults) {
    return new Promise((resolve) => {
        chrome.storage.local.get({ [settingId]: keysAndDefaults }, (obj) => {
            resolve(obj[settingId]);
        });
    });
}


// ストレージに設定を保存する関数
async function setStorage2(settingId, data) {
    return new Promise((resolve) => {
        chrome.storage.local.set({ [settingId]: data }, () => {
            console.log(`Settings saved for ID ${settingId}:`, data);
            resolve();
        });
    });
}


function displayMessage(message, success) {
    const statusMessageElement = document.getElementById("statusMessageForOption");
    statusMessageElement.textContent = message;
    statusMessageElement.style.color = success ? "green" : "red";
    setTimeout(() => {
        statusMessageElement.textContent = "";
    }, 3000);
}


// CSS用
function getGGSG(ggsg) {
    if (ggsg.value == 8) {
        return "ggsg-gi";
    }
    else if (ggsg.value == 4) {
        return "ggsg-go";
    }
    else if (ggsg.value == 2) {
        return "ggsg-shoku";
    }
    else if (ggsg.value == 1) {
        return "ggsg-gun";
    }
    return;
}


// ボタンの移動
// ボタンセットの移動
function moveButtonSet(buttonType, direction, index, maxIndex) {
    //console.log("moveButtonSet index:", index); // ボタン位置
    let cindex = 0;
    if (direction === "up" && index >= 2) {
        cindex = index - 1;
    } else if (direction === "down" && index < maxIndex) {
        cindex = index + 1;
    }
    else {
        console.log("想定外ルート");
        return;
    }

    const orgTextKey = `button${buttonType}${index}Text`;
    const orgText = document.getElementById(orgTextKey).value;
    const orgUrlKey = `button${buttonType}${index}Url`;
    const orgUrl = document.getElementById(orgUrlKey).value;
    const orgPropertyKey = `button${buttonType}${index}Property`;
    const orgProperty = document.getElementById(orgPropertyKey).value;

    const changedTextKey = `button${buttonType}${cindex}Text`;
    const changedText = document.getElementById(changedTextKey).value;
    const changedUrlKey = `button${buttonType}${cindex}Url`;
    const changedUrl = document.getElementById(changedUrlKey).value;
    const changedPropertyKey = `button${buttonType}${cindex}Property`;
    const changedProperty = document.getElementById(changedPropertyKey).value;

    document.getElementById(changedTextKey).value = orgText;
    document.getElementById(orgTextKey).value = changedText;
    document.getElementById(changedUrlKey).value = orgUrl;
    document.getElementById(orgUrlKey).value = changedUrl;
    document.getElementById(changedPropertyKey).value = orgProperty;
    document.getElementById(orgPropertyKey).value = changedProperty;
}

